/*
 * Copyright 2022 NVIDIA Corporation. All rights reserved.
 *
 * NOTICE TO LICENSEE:
 *
 * This source code and/or documentation ("Licensed Deliverables") are
 * subject to NVIDIA intellectual property rights under U.S. and
 * international Copyright laws.
 *
 * These Licensed Deliverables contained herein is PROPRIETARY and
 * CONFIDENTIAL to NVIDIA and is being provided under the terms and
 * conditions of a form of NVIDIA software license agreement by and
 * between NVIDIA and Licensee ("License Agreement") or electronically
 * accepted by Licensee. Notwithstanding any terms or conditions to
 * the contrary in the License Agreement, reproduction or disclosure
 * of the Licensed Deliverables to any third party without the express
 * written consent of NVIDIA is prohibited.
 *
 * NOTWITHSTANDING ANY TERMS OR CONDITIONS TO THE CONTRARY IN THE
 * LICENSE AGREEMENT, NVIDIA MAKES NO REPRESENTATION ABOUT THE
 * SUITABILITY OF THESE LICENSED DELIVERABLES FOR ANY PURPOSE. IT IS
 * PROVIDED "AS IS" WITHOUT EXPRESS OR IMPLIED WARRANTY OF ANY KIND.
 * NVIDIA DISCLAIMS ALL WARRANTIES WITH REGARD TO THESE LICENSED
 * DELIVERABLES, INCLUDING ALL IMPLIED WARRANTIES OF MERCHANTABILITY,
 * NONINFRINGEMENT, AND FITNESS FOR A PARTICULAR PURPOSE.
 * NOTWITHSTANDING ANY TERMS OR CONDITIONS TO THE CONTRARY IN THE
 * LICENSE AGREEMENT, IN NO EVENT SHALL NVIDIA BE LIABLE FOR ANY
 * SPECIAL, INDIRECT, INCIDENTAL, OR CONSEQUENTIAL DAMAGES, OR ANY
 * DAMAGES WHATSOEVER RESULTING FROM LOSS OF USE, DATA OR PROFITS,
 * WHETHER IN AN ACTION OF CONTRACT, NEGLIGENCE OR OTHER TORTIOUS
 * ACTION, ARISING OUT OF OR IN CONNECTION WITH THE USE OR PERFORMANCE
 * OF THESE LICENSED DELIVERABLES.
 *
 * U.S. Government End Users. These Licensed Deliverables are a
 * "commercial item" as that term is defined at 48 C.F.R. 2.101 (OCT
 * 1995), consisting of "commercial computer software" and "commercial
 * computer software documentation" as such terms are used in 48
 * C.F.R. 12.212 (SEPT 1995) and is provided to the U.S. Government
 * only as a commercial end item. Consistent with 48 C.F.R.12.212 and
 * 48 C.F.R. 227.7202-1 through 227.7202-4 (JUNE 1995), all
 * U.S. Government End Users acquire the Licensed Deliverables with
 * only those rights set forth herein.
 *
 * Any use of the Licensed Deliverables in individual and commercial
 * software must include, in the user documentation and internal
 * comments to the code, the above Disclaimer and U.S. Government End
 * Users Notice.
 */

/**
 * @file ORB.h
 *
 * Declares functions that implement support for ORB.
 */

#ifndef NV_VPI_ALGORITHMS_ORB_H
#define NV_VPI_ALGORITHMS_ORB_H

/**
 * @defgroup VPI_OrbFeatureDetector ORB feature
 * @ingroup VPI_API_Algorithms
 * @{
 *
 * Runs the orb algorithm over the input image.
 * Refer to \ref algo_orb_feature_detector for more details and usage examples.
 */

#include "../Export.h"
#include "../Status.h"
#include "../Types.h"
#include "FASTCorners.h"

#include <stddef.h>
#include <stdint.h>

#ifdef __cplusplus
extern "C" {
#endif

/**
 * Structure that defines the parameters for both \ref vpiCreateORBFeatureDetector and \ref vpiSubmitORBFeatureDetector.
 */
typedef struct
{
    /**
     * Parameters for oFAST implementation.
     */
    VPIFASTCornerDetectorParams fastParams;

    /**
     * The maximum number of features alloted per level of the scale pyramid.
     */
    int32_t maxFeatures;

    /**
     * The number of levels in the scale pyramid to utilize.
     */
    int32_t pyramidLevels;

    /**
     * Whether or not to enable rBRIEF (rotationally invariant BRIEF). Disabling this makes it non-rotationally invariant but calculated faster.
     */
    uint8_t enableRBRIEF;

    /**
     * The scoring criteria for ordering the top N FAST corners. Defaults to Harris Corner value. 
     */
    VPICornerScore scoreType;
} VPIORBParams;

/** Initializes VPIORBParams with default values.
 *
 * Default values are:
 * - fastParams           \ref VPIFASTCornerDetectorParams defaults
 * - maxFeatures:         20
 * - scaleFactor:         1.5
 * - pyramidLevels:       4
 * - scoreType:           VPICornerScore::VPI_CORNER_SCORE_HARRIS
 * 
 * @retval #VPI_ERROR_INVALID_ARGUMENT \p params is NULL.
 * @retval #VPI_SUCCESS Operation executed successfully.
 */
VPI_PUBLIC VPIStatus vpiInitORBParams(VPIORBParams *params);

/**
 * Creates a \ref algo_orb_feature_detector "ORB" payload.
 * This function allocates all temporary memory needed by the algorithm.
 *
 * @param[in] backends VPI backends that are eligible to execute the algorithm.
 *                     + Valid values:
 *                       - \ref VPI_BACKEND_CPU
 *                       - \ref VPI_BACKEND_CUDA
 *                     + Backend must be enabled in current context.
 * 
 * @param[in] capacity Capacity of the internal buffer used to store FAST corners (the maximum number of FAST corners detected before filtering).
 *
 * @param[out] payload Pointer to the payload variable that receives the created handle.
 * 
 * @retval #VPI_ERROR_INVALID_ARGUMENT  \p payload is NULL.
 * @retval #VPI_ERROR_INVALID_ARGUMENT  \p inputWidth or \p inputHeight outside valid range.
 * @retval #VPI_ERROR_INVALID_ARGUMENT  \p backends refers to an invalid backend.
 * @retval #VPI_ERROR_INVALID_OPERATION Backend hardware not available.
 * @retval #VPI_ERROR_NOT_IMPLEMENTED   ORB algorithm is not supported by given backend.
 * @retval #VPI_ERROR_INVALID_CONTEXT   Current context is destroyed.
 * @retval #VPI_ERROR_OUT_OF_MEMORY     Cannot allocate required resources.
 * @retval #VPI_ERROR_INVALID_OPERATION Backend isn't enabled in current context.
 * @retval #VPI_SUCCESS                 Operation executed successfully.
 */
VPI_PUBLIC VPIStatus vpiCreateORBFeatureDetector(uint64_t backends, size_t capacity, VPIPayload *payload);

/**
 * Submits a \ref algo_orb_feature_detector "ORB" operation to the stream.
 *
 * @param[in] stream The stream where the operation will be queued in.
 *                   + Must not be NULL.
 *                   + Stream must have enabled the backends that will execute the algorithm.
 *
 * @param[in] backend Backend that will execute the algorithm.
 *                    + Valid values: 
 *                      - \ref VPI_BACKEND_CPU
 *                      - \ref VPI_BACKEND_CUDA
 *
 * @param[in] payload Payload to be submitted along the other parameters.
 * 
 * @param[in] input Input Image Pyramid on which ORB will be executed.
 *                  + Must not be NULL.
 *                  + Image must have enabled the backends that will execute the algorithm.
 *                  + The accepted image formats are:
 *                    - \ref VPI_IMAGE_FORMAT_Y8_ER
 *                    - \ref VPI_IMAGE_FORMAT_Y16_ER
 *                    - \ref VPI_IMAGE_FORMAT_U8
 *                    - \ref VPI_IMAGE_FORMAT_S8
 *                    - \ref VPI_IMAGE_FORMAT_U16
 *                    - \ref VPI_IMAGE_FORMAT_S16
 *
 * @param[out] outCorners Array that will receive the detected corners.
 *                        Array size is updated with the number of corners found.
 *                        + Must not be NULL.
 *                        + It must have type \ref VPI_ARRAY_TYPE_KEYPOINT_F32.
 *                        + Array must have enabled the backends that will execute the algorithm.
 * 
 * @param[out] outDescriptors Array that will receive the descriptors for the corners
 *                            Array size is updated with the number of corners found.
 *                            + Must not be NULL.
 *                            + It must have type \ref VPI_ARRAY_TYPE_KEYPOINT_F32.
 *                            + Array must have enabled the backends that will execute the algorithm.
 *
 * @param[in] params Pointer to a VPIORBParams.
 *                   It defines the parameters for this algorithm invocation.
 *                   These parameters can vary in every call and will be copied internally.
 *                   Thus there is no need to keep the parameters object around.
 *                   - If NULL, use the defaults given by VPIORBParams.
 *
 * @param[in] border How to handle pixels outside image boundaries.
 *                   + The accepted border extensions are:
 *                     - \ref VPI_BORDER_ZERO
 *                     - \ref VPI_BORDER_CLAMP
 *                     - \ref VPI_BORDER_REFLECT
 *                     - \ref VPI_BORDER_MIRROR
 *                     - \ref VPI_BORDER_LIMITED (ignore pixels with circle going outside image boundaries)
 *  
 * @retval #VPI_ERROR_INVALID_ARGUMENT     \p stream is NULL.
 * @retval #VPI_ERROR_INVALID_ARGUMENT     \p input or \p outCorners are NULL.
 * @retval #VPI_ERROR_INVALID_ARGUMENT     Circle radius in \p params outside valid range.
 * @retval #VPI_ERROR_INVALID_ARGUMENT     Arc length in \p params outside valid range.
 * @retval #VPI_ERROR_INVALID_ARGUMENT     Intensity threshold in \p params outside valid range.
 * @retval #VPI_ERROR_INVALID_ARRAY_TYPE   Invalid \p outCorners array type.
 * @retval #VPI_ERROR_INVALID_IMAGE_FORMAT Unsupported input format.
 * @retval #VPI_ERROR_INVALID_OPERATION    The needed backends aren't enabled in \p stream, \p input or \p outCorners.
 * @retval #VPI_SUCCESS                    Operation executed successfully.
 */
VPI_PUBLIC VPIStatus vpiSubmitORBFeatureDetector(VPIStream stream, uint64_t backend, VPIPayload payload,
                                                 VPIPyramid input, VPIArray outCorners, VPIArray outDescriptors,
                                                 const VPIORBParams *params, VPIBorderExtension border);

#ifdef __cplusplus
}
#endif

/** @} */ // end of VPI_OrbFeatureDetector

#endif /* NV_VPI_ALGORITHMS_ORB_H */
