/*
 * Copyright 2021 NVIDIA Corporation. All rights reserved.
 *
 * NOTICE TO LICENSEE:
 *
 * This source code and/or documentation ("Licensed Deliverables") are
 * subject to NVIDIA intellectual property rights under U.S. and
 * international Copyright laws.
 *
 * These Licensed Deliverables contained herein is PROPRIETARY and
 * CONFIDENTIAL to NVIDIA and is being provided under the terms and
 * conditions of a form of NVIDIA software license agreement by and
 * between NVIDIA and Licensee ("License Agreement") or electronically
 * accepted by Licensee. Notwithstanding any terms or conditions to
 * the contrary in the License Agreement, reproduction or disclosure
 * of the Licensed Deliverables to any third party without the express
 * written consent of NVIDIA is prohibited.
 *
 * NOTWITHSTANDING ANY TERMS OR CONDITIONS TO THE CONTRARY IN THE
 * LICENSE AGREEMENT, NVIDIA MAKES NO REPRESENTATION ABOUT THE
 * SUITABILITY OF THESE LICENSED DELIVERABLES FOR ANY PURPOSE. IT IS
 * PROVIDED "AS IS" WITHOUT EXPRESS OR IMPLIED WARRANTY OF ANY KIND.
 * NVIDIA DISCLAIMS ALL WARRANTIES WITH REGARD TO THESE LICENSED
 * DELIVERABLES, INCLUDING ALL IMPLIED WARRANTIES OF MERCHANTABILITY,
 * NONINFRINGEMENT, AND FITNESS FOR A PARTICULAR PURPOSE.
 * NOTWITHSTANDING ANY TERMS OR CONDITIONS TO THE CONTRARY IN THE
 * LICENSE AGREEMENT, IN NO EVENT SHALL NVIDIA BE LIABLE FOR ANY
 * SPECIAL, INDIRECT, INCIDENTAL, OR CONSEQUENTIAL DAMAGES, OR ANY
 * DAMAGES WHATSOEVER RESULTING FROM LOSS OF USE, DATA OR PROFITS,
 * WHETHER IN AN ACTION OF CONTRACT, NEGLIGENCE OR OTHER TORTIOUS
 * ACTION, ARISING OUT OF OR IN CONNECTION WITH THE USE OR PERFORMANCE
 * OF THESE LICENSED DELIVERABLES.
 *
 * U.S. Government End Users. These Licensed Deliverables are a
 * "commercial item" as that term is defined at 48 C.F.R. 2.101 (OCT
 * 1995), consisting of "commercial computer software" and "commercial
 * computer software documentation" as such terms are used in 48
 * C.F.R. 12.212 (SEPT 1995) and is provided to the U.S. Government
 * only as a commercial end item. Consistent with 48 C.F.R.12.212 and
 * 48 C.F.R. 227.7202-1 through 227.7202-4 (JUNE 1995), all
 * U.S. Government End Users acquire the Licensed Deliverables with
 * only those rights set forth herein.
 *
 * Any use of the Licensed Deliverables in individual and commercial
 * software must include, in the user documentation and internal
 * comments to the code, the above Disclaimer and U.S. Government End
 * Users Notice.
 */

/**
 * @file OpticalFlowDense.h
 *
 * Declares functions that implement the dense optical flow.
 */

#ifndef NV_VPI_ALGORITHMS_OPTICALFLOWDENSE_H
#define NV_VPI_ALGORITHMS_OPTICALFLOWDENSE_H

/**
 * @defgroup VPI_OpticalFlowDense Dense Optical Flow
 * @ingroup VPI_API_Algorithms
 * @{
 *
 * Algorithm used to estimate the motion vectors from previous image to current image.
 * Refer to \ref algo_optflow_dense for more details and usage examples.
 */

#include "../Export.h"
#include "../ImageFormat.h"
#include "../Status.h"
#include "../Types.h"

#include <stdbool.h>
#include <stdint.h>

#ifdef __cplusplus
extern "C" {
#endif

/**
 * Creates payload for \ref vpiSubmitOpticalFlowDense
 *
 * @param[in] backends VPI backends that are eligible to execute the algorithm.
 *                     + Supported backends:
 *                       - \ref VPI_BACKEND_NVENC
 *                     + Backend must be enabled in current context.
 *
 * @param[in] width,height Dimensions of current and previous image.
 *                         + Must be >= 0.
 *
 * @param[in] inputFmt Format of current and previous image.
 *                     + The accepted image formats are:
 *                       - \ref VPI_IMAGE_FORMAT_NV12_BL
 *                       - \ref VPI_IMAGE_FORMAT_NV12_ER_BL
 *
 * @param[in] quality Quality of the dense optical flow algorithm.
 *
 * @param[out] payload Pointer to the payload variable that receives the created handle.
 *
 * @retval #VPI_IMAGE_FORMAT_INVALID    \p inputFmt is not supported.
 * @retval #VPI_ERROR_INVALID_ARGUMENT  \p payload is NULL.
 * @retval #VPI_ERROR_INVALID_ARGUMENT  \p width or \p height outside valid range.
 * @retval #VPI_ERROR_INVALID_ARGUMENT  \p backends refers to an invalid backend.
 * @retval #VPI_ERROR_INVALID_OPERATION Backend hardware not available.
 * @retval #VPI_ERROR_INVALID_OPERATION Backend isn't enabled in current context.
 * @retval #VPI_ERROR_NOT_IMPLEMENTED   Dense Optical Flow algorithm is not supported by given backends.
 * @retval #VPI_ERROR_INVALID_CONTEXT   Current context is destroyed.
 * @retval #VPI_ERROR_OUT_OF_MEMORY     Cannot allocate required resources.
 * @retval #VPI_SUCCESS                 Operation executed successfully.
 */
VPI_PUBLIC VPIStatus vpiCreateOpticalFlowDense(uint64_t backends, int32_t width, int32_t height,
                                               VPIImageFormat inputFmt, VPIOpticalFlowQuality quality,
                                               VPIPayload *payload);

/**
 * Runs dense Optical Flow on two frames. Outputs motion vectors for every 4x4 pixel block in the input image.
 *
 * @param[in] stream The stream where the operation will be queued in.
 *                   + Must not be NULL.
 *                   + Stream must have enabled the backends that will execute the algorithm.
 *
 * @param[in] backend Backend that will execute the algorithm.
 *                    + Must be the backend specified during payload creation or 0 as a shorthand to use this backend.
 *
 * @param[in] payload Payload created by \ref vpiCreateOpticalFlowDense. 
 *
 * @param[in] prevImg Previous frame.
 *                    + Must not be NULL.
 *                    + Must have same format and dimensions as the one specified during payload creation.
 *                    + Image must have enabled the backends that will execute the algorithm.
 *
 * @param[in] curImg Current frame.
 *                   + Must not be NULL.
 *                   + Must have same format and dimensions as \p prevImg.
 *                   + Image must have enabled the backends that will execute the algorithm.
 *
 * @param[out] mvImg Motion vectors output.
 *                   + Must not be NULL.
 *                   + Image must have enabled the backends that will execute the algorithm.
 *                   + On NVENC, dimensions must be 1/4 of \p curImg.
 *                   + Supported formats:
 *                     - \ref VPI_IMAGE_FORMAT_2S16_BL is supported. Motion vector is in fixed point S10.5 format.
 * 
 * @retval #VPI_ERROR_INVALID_ARGUMENT     \p stream is NULL.
 * @retval #VPI_ERROR_INVALID_ARGUMENT     \p prevImg, \p curImg or \p mvImg are NULL.
 * @retval #VPI_ERROR_INVALID_ARGUMENT     \p payload is not generated using vpiCreateOpticalFlowDense.
 * @retval #VPI_ERROR_INVALID_ARGUMENT     \p prevImg and \p curImg dimensions does not match the one associated with the \p payload. 
 * @retval #VPI_ERROR_INVALID_IMAGE_FORMAT \p prevImg and \p curImg format does not match the one associated with the \p payload. 
 * @retval #VPI_ERROR_INVALID_PAYLOAD_TYPE \p payload is invalid.
 * @retval #VPI_ERROR_INVALID_OPERATION    The needed backends aren't enabled in \p stream, \p prevImg, \p curImg or \p mvImg.
 * @retval #VPI_SUCCESS                    Operation executed successfully.
 */
VPI_PUBLIC VPIStatus vpiSubmitOpticalFlowDense(VPIStream stream, uint64_t backend, VPIPayload payload, VPIImage prevImg,
                                               VPIImage curImg, VPIImage mvImg);

#ifdef __cplusplus
}
#endif

/** @} */ // end of VPI_OpticalFlowDense

#endif // NV_VPI_ALGORITHMS_OPTICALFLOWDENSE_H
