/*
 * Copyright 2019-2022 NVIDIA Corporation. All rights reserved.
 *
 * NOTICE TO LICENSEE:
 *
 * This source code and/or documentation ("Licensed Deliverables") are
 * subject to NVIDIA intellectual property rights under U.S. and
 * international Copyright laws.
 *
 * These Licensed Deliverables contained herein is PROPRIETARY and
 * CONFIDENTIAL to NVIDIA and is being provided under the terms and
 * conditions of a form of NVIDIA software license agreement by and
 * between NVIDIA and Licensee ("License Agreement") or electronically
 * accepted by Licensee. Notwithstanding any terms or conditions to
 * the contrary in the License Agreement, reproduction or disclosure
 * of the Licensed Deliverables to any third party without the express
 * written consent of NVIDIA is prohibited.
 *
 * NOTWITHSTANDING ANY TERMS OR CONDITIONS TO THE CONTRARY IN THE
 * LICENSE AGREEMENT, NVIDIA MAKES NO REPRESENTATION ABOUT THE
 * SUITABILITY OF THESE LICENSED DELIVERABLES FOR ANY PURPOSE. IT IS
 * PROVIDED "AS IS" WITHOUT EXPRESS OR IMPLIED WARRANTY OF ANY KIND.
 * NVIDIA DISCLAIMS ALL WARRANTIES WITH REGARD TO THESE LICENSED
 * DELIVERABLES, INCLUDING ALL IMPLIED WARRANTIES OF MERCHANTABILITY,
 * NONINFRINGEMENT, AND FITNESS FOR A PARTICULAR PURPOSE.
 * NOTWITHSTANDING ANY TERMS OR CONDITIONS TO THE CONTRARY IN THE
 * LICENSE AGREEMENT, IN NO EVENT SHALL NVIDIA BE LIABLE FOR ANY
 * SPECIAL, INDIRECT, INCIDENTAL, OR CONSEQUENTIAL DAMAGES, OR ANY
 * DAMAGES WHATSOEVER RESULTING FROM LOSS OF USE, DATA OR PROFITS,
 * WHETHER IN AN ACTION OF CONTRACT, NEGLIGENCE OR OTHER TORTIOUS
 * ACTION, ARISING OUT OF OR IN CONNECTION WITH THE USE OR PERFORMANCE
 * OF THESE LICENSED DELIVERABLES.
 *
 * U.S. Government End Users. These Licensed Deliverables are a
 * "commercial item" as that term is defined at 48 C.F.R. 2.101 (OCT
 * 1995), consisting of "commercial computer software" and "commercial
 * computer software documentation" as such terms are used in 48
 * C.F.R. 12.212 (SEPT 1995) and is provided to the U.S. Government
 * only as a commercial end item. Consistent with 48 C.F.R.12.212 and
 * 48 C.F.R. 227.7202-1 through 227.7202-4 (JUNE 1995), all
 * U.S. Government End Users acquire the Licensed Deliverables with
 * only those rights set forth herein.
 *
 * Any use of the Licensed Deliverables in individual and commercial
 * software must include, in the user documentation and internal
 * comments to the code, the above Disclaimer and U.S. Government End
 * Users Notice.
 */

#ifndef NV_VPI_ALGORITHMS_STEREO_DISPARITY_H
#define NV_VPI_ALGORITHMS_STEREO_DISPARITY_H

/**
 * @file StereoDisparity.h
 *
 * Declares functions that implement stereo disparity estimation algorithms.
 */

#include "../Export.h"
#include "../ImageFormat.h"
#include "../Status.h"
#include "../Types.h"

#include <stdint.h>

#ifdef __cplusplus
extern "C" {
#endif

/**
 * @defgroup VPI_StereoDisparityEstimator Stereo Disparity Estimator
 * @ingroup VPI_API_Algorithms
 * @{
 *
 * Estimates disparity from a stereo pair.
 * Refer to \ref algo_stereo_disparity for more details and usage examples.
 */

/**
 * Structure that defines the parameters for \ref vpiCreateStereoDisparityEstimator
 */
typedef struct
{
    /** Maximum disparity for matching search.
     *  + Must be positive.
     *  + On PVA backend, maxDisparity must be == 64.
     *  + On OFA or OFA+PVA+VIC backend, maxDisparity must be 128 or 256.
     *  + On CUDA backends, maxDisparity must be >= 0 and <= 256.
     *  + On PVA+NVENC+VIC backends, range from >= 0 and <= 256 is accepted, but
     *    internally this value is ignored and 256 is used instead.
     */
    int32_t maxDisparity;

    /** Output's downscale factor with respect to the input's resolution.
     *  + For PVA+NVENC+VIC backend, only 4 is supported.
     *    Setting it to 4 leads to faster execution speeds as NVENC engine internal output has 1/4th resolution.
     *  + For OFA and OFA+PVA+VIC backend, the allowed values are 1, 2, 4 or 8.
     *  + For other backends, only 1 is supported, i.e. output dimension equals input's. */
    int32_t downscaleFactor;

} VPIStereoDisparityEstimatorCreationParams;

/**
 * Initializes \ref VPIStereoDisparityEstimatorCreationParams with default values.
 *
 * Defaults:
 * - maxDisparity    = 64
 * - downscaleFactor = 1
 *
 * @param[in] params Structure to be filled with default values.
 *
 * @retval #VPI_ERROR_INVALID_ARGUMENT \p params is NULL.
 * @retval #VPI_SUCCESS Operation executed successfully.
 */
VPI_PUBLIC VPIStatus vpiInitStereoDisparityEstimatorCreationParams(VPIStereoDisparityEstimatorCreationParams *params);

/**
 * Creates payload for \ref vpiSubmitStereoDisparityEstimator
 *
 * @param[in] backends VPI backends that are eligible to execute the algorithm.
 *                     + Valid values:
 *                       - \ref VPI_BACKEND_CPU
 *                       - \ref VPI_BACKEND_CUDA
 *                       - \ref VPI_BACKEND_OFA
 *                       - \ref VPI_BACKEND_PVA | \ref VPI_BACKEND_OFA   | \ref VPI_BACKEND_VIC
 *                       - \ref VPI_BACKEND_PVA | \ref VPI_BACKEND_NVENC | \ref VPI_BACKEND_VIC
 *                     + Backends must be enabled in current context.
 *
 * @param[in] imageWidth, imageHeight Input image dimensions.
 *                                    + Must be >= 0.
 *                                    + On PVA, input dimensions must be 480x270.
 *                                    + On PVA+NVENC+VIC, input dimensions must be 1920x1080.
 *                                    + On OFA+PVA+VIC, input width must be at least max(64, maxDisparity/downscaleFactor) * downscaleFactor.
 *
 * @param[in] inputFormat Input image format.
 *                        + Supported image formats are:
 *                          | Formats                          | CUDA | CPU | PVA | OFA | OFA+PVA+VIC | PVA+NVENC+VIC |
 *                          |----------------------------------|:----:|:---:|:---:|:---:|:-----------:|:-------------:|
 *                          | \ref VPI_IMAGE_FORMAT_U8         |   *  |  *  |     |     |             |               |
 *                          | \ref VPI_IMAGE_FORMAT_U16        |   *  |  *  |  *  |     |             |               |
 *                          | \ref VPI_IMAGE_FORMAT_Y8         |   *  |  *  |     |     |             |               |
 *                          | \ref VPI_IMAGE_FORMAT_Y8_BL      |      |     |     |  *  |      *      |               |
 *                          | \ref VPI_IMAGE_FORMAT_Y8_ER      |   *  |  *  |     |     |             |               |
 *                          | \ref VPI_IMAGE_FORMAT_Y8_ER_BL   |      |     |     |  *  |      *      |               |
 *                          | \ref VPI_IMAGE_FORMAT_Y16        |   *  |  *  |     |     |             |               |
 *                          | \ref VPI_IMAGE_FORMAT_Y16_BL     |      |     |     |  *  |      *      |               |
 *                          | \ref VPI_IMAGE_FORMAT_Y16_ER     |   *  |  *  |  *  |     |             |               |
 *                          | \ref VPI_IMAGE_FORMAT_Y16_ER_BL  |      |     |     |  *  |      *      |       *       |
 *                          | \ref VPI_IMAGE_FORMAT_NV12       |   *  |  *  |     |     |             |               |
 *                          | \ref VPI_IMAGE_FORMAT_NV12_BL    |      |     |     |  *  |      *      |               |
 *                          | \ref VPI_IMAGE_FORMAT_NV12_ER    |   *  |  *  |     |     |             |               |
 *                          | \ref VPI_IMAGE_FORMAT_NV12_ER_BL |      |     |     |  *  |      *      |               |
 *                          | \ref VPI_IMAGE_FORMAT_NV24       |   *  |  *  |     |     |             |               |
 *                          | \ref VPI_IMAGE_FORMAT_NV24_BL    |      |     |     |  *  |      *      |               |
 *                          | \ref VPI_IMAGE_FORMAT_NV24_ER    |   *  |  *  |     |     |             |               |
 *                          | \ref VPI_IMAGE_FORMAT_NV24_ER_BL |      |     |     |  *  |      *      |               |
 *
 * @param[in] params Creation parameters.
 *                   Pass NULL to use defaults given by \ref vpiInitStereoDisparityEstimatorCreationParams.
 *
 * @param[out] payload Pointer to the payload variable that receives the created handle.
 *
 * @retval #VPI_IMAGE_FORMAT_INVALID    \p inputFormat is not supported.
 * @retval #VPI_ERROR_INVALID_ARGUMENT  \p payload is NULL.
 * @retval #VPI_ERROR_INVALID_ARGUMENT  \p imageWidth or \p imageHeight outside valid range.
 * @retval #VPI_ERROR_INVALID_ARGUMENT  \p backends refers to an invalid backend.
 * @retval #VPI_ERROR_INVALID_ARGUMENT  Maximum disparity in \p params outside valid range.
 * @retval #VPI_ERROR_NOT_IMPLEMENTED   Stereo Disparity Estimator algorithm is not supported by given backends.
 * @retval #VPI_ERROR_INVALID_OPERATION Backend hardware not available.
 * @retval #VPI_ERROR_INVALID_CONTEXT   Current context is destroyed.
 * @retval #VPI_ERROR_OUT_OF_MEMORY     Cannot allocate required resources.
 * @retval #VPI_ERROR_INVALID_OPERATION Backend isn't enabled in current context.
 * @retval #VPI_SUCCESS                 Operation executed successfully.
 */
VPI_PUBLIC VPIStatus vpiCreateStereoDisparityEstimator(uint64_t backends, int32_t imageWidth, int32_t imageHeight,
                                                       VPIImageFormat inputFormat,
                                                       const VPIStereoDisparityEstimatorCreationParams *params,
                                                       VPIPayload *payload);

/**
 * Structure that defines the parameters for \ref vpiSubmitStereoDisparityEstimator
 */
typedef struct
{
    /** Represents the median filter size (on PVA+NVENC+VIC or OFA+PVA+VIC backend)
        or census transform window size (other backends) used in the algorithm.
     *  + On PVA backend, it must be 5.
     *  + On CPU backend, it must be >= 1.
     *  + On CUDA backend this is ignored. A 9x7 window is used instead.
     *  + On OFA backend it is ignored.
     *  + On OFA+PVA+VIC backend, valid values are 1, 3, 5 or 7.
     *  + On PVA+NVENC+VIC backend, valid values are 1, 3, 5 or 7. */
    int32_t windowSize;

    /** Maximum disparity for matching search.
     *  + Maximum disparity must be 0 (use from payload), or positive and less or equal to what's configured in payload. */
    int32_t maxDisparity;

    /* Confidence threshold above which disparity values are considered valid.
     * Only used in CUDA, PVA+NVENC+VIC, and OFA+PVA+VIC backends.
     * + Ranges from 1 to 65280, values outside this range get clamped. */
    int32_t confidenceThreshold;

    /* Quality of disparity output.
     * It's only applicable when using PVA+NVENC+VIC backend.
     * The higher the value, better the quality and possibly slower perf.
     * + Must be a value between 1 and 8. */
    int32_t quality;

} VPIStereoDisparityEstimatorParams;

/**
 * Initializes \ref VPIStereoDisparityEstimatorParams with default values.
 *
 * Defaults:
 * - windowSize          = 5
 * - maxDisparity        = 0 (uses disparity set during payload construction)
 * - confidenceThreshold = 32767
 * - quality             = 6
 *
 * @param[in] params Structure to be filled with default values.
 *
 * @retval #VPI_ERROR_INVALID_ARGUMENT \p params is NULL.
 * @retval #VPI_SUCCESS Operation executed successfully.
 */
VPI_PUBLIC VPIStatus vpiInitStereoDisparityEstimatorParams(VPIStereoDisparityEstimatorParams *params);

/**
 * Runs stereo processing on a pair of images and outputs a disparity map.
 *
 * @param[in] stream The stream where the operation will be queued in.
 *                   + Must not be NULL.
 *                   + Stream must have enabled the backends that will execute the algorithm.
 *
 * @param[in] backend Backend that will execute the algorithm.
 *                    + Must be the backend specified during payload creation or 0 as a shorthand to use this backend.
 *
 * @param[in] payload Payload to be submitted along the other parameters.
 *
 * @param[in] left Left stereo input image.
 *                 + Must not be NULL.
 *                 + Must have same format and dimensions as the one specified during payload creation.
 *                 + Image must have enabled the backends that will execute the algorithm.
 *
 * @param[in] right Right stereo input image.
 *                  + Must not be NULL.
 *                  + Must have same format and dimensions as \p left.
 *                  + Image must have enabled the backends that will execute the algorithm.
 *
 * @param[out] disparity Image where the disparity values will be written to.
 *                       Returned values are in Q10.5 format, i.e., signed fixed point with 5 fractional bits.
 *                       Divide it by 32.0f to convert it to floating point.
 *                       + Must not be NULL.
 *                       + On OFA backend, image format must be \ref VPI_IMAGE_FORMAT_S16_BL
 *                       + On other backends, image format must be \ref VPI_IMAGE_FORMAT_S16.
 *                       + On OFA+PVA+VIC backend, and if windowSize==1, confidenceMap==NULL and confidenceThreshold==0,
 *                         the image format can be either \ref VPI_IMAGE_FORMAT_S16 or \ref VPI_IMAGE_FORMAT_S16_BL.
 *                       + Dimensions must be equal to \p input,
 *                         but downscaled by \ref VPIStereoDisparityEstimatorCreationParams::downscaleFactor
 *                         passed in \p params.
 *                       + Image must have enabled the backends that will execute the algorithm.
 *
 * @param[out] confidenceMap Image containing a confidence score telling how accurate the disparity is.
 *                           Pass NULL if it isn't needed.
 *                           Returned confidence range from 0 to 65535.
 *                           The higher the confidence, more accurate is the corresponding disparity.
 *                           + CPU, PVA and OFA backends don't support confidence map output, it must be NULL.
 *                           + Image format must be \ref VPI_IMAGE_FORMAT_U16
 *                           + Must be same dimensions as \p disparity.
 *                           + Image must have enabled the backends that will execute the algorithm.
 *
 * @param[in] params Pointer to algorithm control parameters. If NULL, it'll use the
 *                   defaults given by \ref vpiInitStereoDisparityEstimatorParams.
 *
 * @retval #VPI_ERROR_INVALID_ARGUMENT     \p stream is NULL.
 * @retval #VPI_ERROR_INVALID_ARGUMENT     \p left, \p right or \p disparity are NULL.
 * @retval #VPI_ERROR_INVALID_ARGUMENT     Maximum disparity in \p params outside valid range.
 * @retval #VPI_ERROR_INVALID_ARGUMENT     Window size in \p params outside valid range.
 * @retval #VPI_ERROR_INVALID_ARGUMENT     \p left and \p right images must have the same dimensions.
 * @retval #VPI_ERROR_INVALID_ARGUMENT     Input image dimensions must be equal to what's configured in \p payload.
 * @retval #VPI_ERROR_INVALID_ARGUMENT     \p confidenceMap and \p disparity must have same dimensions.
 * @retval #VPI_ERROR_INVALID_ARGUMENT     Backend doesn't output confidence map.
 * @retval #VPI_ERROR_INVALID_IMAGE_FORMAT \p left and \p right format don't match payload image format.
 * @retval #VPI_ERROR_INVALID_IMAGE_FORMAT \p disparity format is invalid.
 * @retval #VPI_ERROR_INVALID_PAYLOAD_TYPE \p payload is invalid.
 * @retval #VPI_ERROR_INVALID_OPERATION    The needed backends aren't enabled in \p stream, \p left, \p right,
 *                                         \p disparity or \p confidenceMap.
 * @retval #VPI_SUCCESS                    Operation executed successfully.
 */
VPI_PUBLIC VPIStatus vpiSubmitStereoDisparityEstimator(VPIStream stream, uint64_t backend, VPIPayload payload,
                                                       VPIImage left, VPIImage right, VPIImage disparity,
                                                       VPIImage confidenceMap,
                                                       const VPIStereoDisparityEstimatorParams *params);

/** @} end of VPI_StereoDisparityEstimator */

#ifdef __cplusplus
}
#endif

#endif /* NV_VPI_ALGORITHMS_STEREO_DISPARITY_H */
